/* ============================= C MeatAxe ==================================
   File:        $Id: fpmul2.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Print a factored polynomial.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function FpMul "Multiply factored polynomials"
!synopsis 
    FPoly_t *FpMul(FPoly_t *dest, const FPoly_t *src);
!arg dest
    Factored polynomial to modify.
!arg src
    Factored polynomial.
!return
    The function returns |dest| or |NULL| on error.
!description
    Multiplies the factored polynomial |dest| by the factored 
    polynomial |src|.
!seealso FpMulP
-------------------------------------------------------------------------- }}}*/

FPoly_t *FpMul(FPoly_t *dest, const FPoly_t *src)

{
    int i;

    /* Check the arguments
       ------------------- */
    if (!FpIsValid(src) || !FpIsValid(dest))
	return NULL;

    for (i = 0; i < src->NFactors; ++i)
    {
	if (FpMulP(dest,src->Factor[i],src->Mult[i]) == NULL)
	{
	    MTX_ERROR("Cannot multiply");
	    return NULL;
	}
    }
    return dest;
}

