/* ============================= C MeatAxe ==================================
   File:        $Id: imatread.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Read an integer matrix from a file.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.imat
!function ImatRead "Read an integer matrix from a file"
!arg f
    File to read from.
!return
    Pointer to the matrix, or |NULL| on error.
!description
    This function reads an integer matrix from a file.
!seealso 
-------------------------------------------------------------------------- }}}*/

IntMatrix_t *ImatRead(FILE *f)

{
    IntMatrix_t *m;
    long hdr[3];

    if (SysReadLong(f,hdr,3) != 3) 
    {
	MTX_ERROR("Cannot read header");
	return NULL;
    }
    if (hdr[0] != -8)	/* HACK: T_IMAT in 2.3 */
    {
	MTX_ERROR("Not an integer matrix");
	return NULL;
    }
    m = ImatAlloc(hdr[1],hdr[2]);
    if (m == NULL) 
	return NULL;
    if (SysReadLong(f,m->Data,m->Nor*m->Noc) != m->Nor * m->Noc)
    {
	ImatFree(m);
	return NULL;
    }
    return m;
}


/*{{{ --------------------------------------------------------------------------
!function ImatLoad "Read an integer matrix from a file"
!arg fn
    File name.
!return
    Pointer to the matrix, or |NULL| on error.
!description
    This function opens a file, reads a single integer matrix, and closes 
    the file. To read more than one matrix from a file, use |ImatRead()|.
!seealso MatRead
-------------------------------------------------------------------------- }}}*/

IntMatrix_t *ImatLoad(const char *fn)

{
    FILE *f;
    IntMatrix_t *m;

    if ((f = SysFopen(fn,FM_READ)) == NULL)
	return NULL;
    m = ImatRead(f);
    fclose(f);
    return m;
}

