/* ============================= C MeatAxe ==================================
   File:        $Id: matprint.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Clean a row vector.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"


/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

/*
MTX_DEFINE_FILE_INFO 
*/

/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatPrint "Print a matrix on stdout"
!synopsis 
    void MatPrint(const char *name, const Matrix_t *m);
!arg name
    Name to print before the matrix, or |NULL|.
!arg m
    Pointer to the matrix.
!description
    This function prints a matrix on the standard output in readable form.
    If |name| is not |NULL| the name followed by an equal sign is printed 
    before the matrix.
!seealso
-------------------------------------------------------------------------- }}}*/

void MatPrint(const char *name, const Matrix_t *m)

{
    PTR x;
    long i, k;

    if (!MatIsValid(m)) 
	return;
    FfSetField(m->Field);
    FfSetNoc(m->Noc);
    x = m->Data;
    if (name != NULL) printf("%s=\n",name);
    for (i = 0; i < m->Nor; ++i)
    {
	for (k = 0; k < m->Noc; ++k)
	    printf("%d",FfExtract(x,k));
	printf("\n");
	FfStepPtr(&x);
    }
}

