/* ============================= C MeatAxe ==================================
   File:        $Id: mattrace.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Trace of a matrix
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"



/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatTrace "Trace of a matrix"
!synopsis 
    FEL MatTrace(const Matrix_t *mat);
!arg mat
    Pointer to the matrix.
!return
    Trace of |mat|, |FF_ZERO| on error.
!description
    This function calculates the sum of all diagonal elements of a matrix.
    Note that the matrix need not be square.
!seealso
-------------------------------------------------------------------------- }}}*/

FEL MatTrace(const Matrix_t *mat)

{
    int i;
    PTR x;
    int maxi;
    FEL trace = FF_ZERO;

    /* Check the argument
       ------------------ */
#ifdef DEBUG
    if (!MatIsValid(mat))
	return FF_ZERO;
#endif

    maxi = mat->Nor > mat->Noc ? mat->Noc : mat->Nor;
    FfSetField(mat->Field);
    FfSetNoc(mat->Noc);
    for (i = 0, x = mat->Data; i < maxi; ++i, FfStepPtr(&x))
	trace = FfAdd(trace,FfExtract(x,i));
    return trace;
}
