/* ============================= C MeatAxe ==================================
   File:        $Id: polprint.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Print a polynomial.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO


/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function PolPrint "Print a polynomial"
!arg name
    Name to print before the polynomial or |NULL| (see below).
!arg p
    Pointer to the polynomial.
!return
    $0$ on success, $-1$ on error.
!description
    This function prints a polynomial on the standard output in a human-readable
    form. If |name| is not |NULL| the name followed by an equal sign is printed 
    before the polynomial. For example, the
    statement |PolPrint("P",P)| may produce the following output:
    ||    P=3x^2+x+1
!seealso
-------------------------------------------------------------------------- }}}*/

void PolPrint(char *name, const Poly_t *p)

{
    int i,flag = 0;

    if (!PolIsValid(p))
    {
	MTX_ERROR1("%E",MTX_ERR_BADARG);
	return;
    }
    if (name != NULL) 
	printf("%s=",name);
    FfSetField(p->Field);
    if (p->Degree == -1) 
    {
	printf("0x^0");
    }
    for (i = p->Degree; i >= 0; i--)
    {
	if (p->Data[i] != FF_ZERO)
	{
	    if (flag) 
		printf("+");
	    if (p->Data[i] != FF_ONE || i == 0)
		printf("%d",FfToInt(p->Data[i]));
	    if (i > 1) 
		printf("x^%d",i);
	    else if (i == 1) 
		printf("x");
	    flag=1;
       	}
    }
    if (name != NULL) 
	printf("\n");
}




