/* ============================= C MeatAxe ==================================
   File:        $Id: setprint.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Basic set functions.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO

/*{{{ --------------------------------------------------------------------------
!section obj.set
!function SetPrint "Write a set to stdout"
!synopsis 
    void SetPrint(char *name, const Set_t *s);
!arg name
    Name to print before the set, or |NULL|.
!arg s
    Pointer to the set.
!description
    This function prints a set in human-readable format on the standard 
    output. If |name| is different from |NULL|, |name| and an `=' is printed
    before the set.
!seealso Set_t
-------------------------------------------------------------------------- }}}*/

int SetPrint(char *name, const Set_t *s)

{
    int i;

    if (!SetIsValid(s))
	return -1;
    if (name != NULL) 
	printf("%s=",name);
    printf("{");
    for (i = 0; i < s->Size; ++i)
    {
	if (i > 0) printf(",");
	printf("%ld",s->Data[i]);
    }
    printf("}");
    if (name != NULL) printf("\n");
    fflush(stdout);
    return 0;
}


