/* ============================= C MeatAxe ==================================
   File:        $Id: zpo.c,v 1.3 2001/07/14 10:41:05 mringe Exp $
   Comment:     Power of a matrix or permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

#include <ctype.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

MTX_DEFINE_FILE_INFO

static int Power;

static MtxApplicationInfo_t AppInfo = { 
"power", "Power of a matrix or permutation", 
"SYNTAX\n"
"    zpo [<Options>] <In> <n> <Result>\n"
"\n"
"OPTIONS\n"
MTX_COMMON_OPTIONS_DESCRIPTION
"\n"
"ARGUMENTS\n"
"    <In> ........... Input: Matrix or permutation\n"
"    <n> ............ Power to compute (e.g., 5 or pwr5)\n"
"    <Result> ....... Ouput: <n>-th power of <In>\n"
};

static MtxApplication_t *App = NULL;








/* ------------------------------------------------------------------
   init() - Process command line options and arguments
   ------------------------------------------------------------------ */

static int Init(int argc, const char **argv)

{
    /* Process command line options.
       ----------------------------- */
    App = AppAlloc(&AppInfo,argc,argv);
    if (App == NULL)
	return -1;

    /* Process arguments.
       ------------------ */
    if (AppGetArguments(App,3,3) < 0)
	return -1;
    if (!strncmp(App->ArgV[1],"pwr",3))
        Power = atoi(App->ArgV[1] + 3);		/* ZSM compatibility (pwrN) */
    else
        Power = atoi(App->ArgV[1]);

    return 0;
}




static void Cleanup()

{
    AppFree(App);
}



static int CalcPower()
{
    void *x = XLoad(App->ArgV[0]);
    void *y;

    if (x == NULL)
	return -1;
    y = XPower(x,Power);
    if (y == NULL)
	return -1;
    XSave(y,App->ArgV[2]);
    XFree(x);
    XFree(y);
    return 0;

}






/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, const char **argv)
{
    if (Init(argc,argv) != 0)
    {
	MTX_ERROR("Initialization failed");
	return 1;
    }
    CalcPower();
    Cleanup();
    return EXIT_OK;
}




/*{{{ --------------------------------------------------------------------------
!section apps.general
!program zpo "Power"
!synopsis 
    zpo [<Options>] <In> <n> <Result>
!arg <In>
    Input file, matrix or permutation.
!arg <n>
    The power to calculate. Integer or ZSM format (see below).
!arg <Result>
    Output: n-th power of the matrix or permutation.
!description
    This program calculates the n-th power of a matrix or permutation.
    <n> may be negative. For compatibility with the old ZSM program,
    the power may be specified in the ZSM format. The following example
    shows two equivalent ways to calculate the 69-th power of a matrix:
    ||    zpo matrix 69 result
    ||    zpo matrix pwr69 result
!seealso 
-------------------------------------------------------------------------- }}}*/


